
#pragma once

// Genode includes
#include <base/attached_dataspace.h>
#include <base/mutex.h>
#include <base/rpc_client.h>
#include <util/misc_math.h>
#include <util/string.h>

// gapfruit includes
#include <verify_signature_session/verify_session.h>


namespace Verify_signature
{
	using Genode::Attached_dataspace;
	using Genode::Capability;
	using Genode::Mutex;
	using Genode::Region_map;
	using Genode::Rpc_client;
	using Genode::memcpy;
	using Genode::size_t;

	class Session_client;
}


class Verify_signature::Session_client : public Rpc_client<Verify_signature::Session>
{
	private:

		mutable Mutex _mutex { };

		/**
		 * Shared-memory buffer used for carrying the payload
		 * of read/write operations
		 */
		Attached_dataspace _io_buffer_data;
		Attached_dataspace _io_buffer_certificate;
		Attached_dataspace _io_buffer_signature;

		Signature_algorithm _algo { };

	public:

		Session_client(Region_map &local_rm, Capability<Session> cap) :
			Rpc_client<Session>    { cap },
			_io_buffer_data        { local_rm, call<Rpc_dataspace_data>() },
			_io_buffer_certificate { local_rm, call<Rpc_dataspace_certificate>() },
			_io_buffer_signature   { local_rm, call<Rpc_dataspace_signature>() }
		{ }

		Result verify(const Signature_algorithm algo,
		              const uint8_t* data,        const size_t data_size,
		              const uint8_t* certificate, const size_t certificate_size,
		              const uint8_t* signature,   const size_t signature_size) override
		{
			Mutex::Guard _guard(_mutex);

			if (data_size        > _io_buffer_data.size())        throw Buffer_too_small();
			if (certificate_size > _io_buffer_certificate.size()) throw Buffer_too_small();
			if (signature_size   > _io_buffer_signature.size())   throw Buffer_too_small();

			_algo = algo;

			memcpy(_io_buffer_data.local_addr<char>(),        data,        data_size);
			memcpy(_io_buffer_certificate.local_addr<char>(), certificate, certificate_size);
			memcpy(_io_buffer_signature.local_addr<char>(),   signature,   signature_size);

			return call<Rpc_verify>(algo, data_size, certificate_size, signature_size);
		}
};
